unit Isamtool;

interface

uses
  SysUtils, ActiveX;

{$A+}

const

	VIS_OK = 0;
  VIS_NOT_FOUND = 1;
  VIS_UPDATE_VIOLATION = 2;
	VIS_ACCESS_DENIED = 3;
  VIS_BAD_DATASET_NUMBER = 4;
  VIS_BAD_FORMAT = 5;
	VIS_INVALID_KEY = 6;
  VIS_BAD_PARAMETER_VALUE = 7;
  VIS_BAD_FILE = 8;
	VIS_ALREADY_EXISTS = 9;
  VIS_NO_ROOM = 10;
  VIS_DISK_FULL = 11;
	VIS_OUT_OF_MEMORY = 12;
  VIS_DOS_ERROR = 13;
  VIS_DISK_ERROR = 14;
  VIS_DATA_VALIDITY_CHECK = 15;
  VIS_INVALID_SECONDARY_KEY = 16;
  VIS_SEQUENCE_ERROR = 17;
  VIS_OUT_OF_FILE_HANDLES = 18;
  VIS_BUSY = 19;
  VIS_INVALID_PASSWORD = 20;
  VIS_FUNCTION_NOT_AVAILABLE = 21;
  VIS_INTERUPTED = 22;
  VIS_INCORRECT_PASSWORD = 23;

  ADD_OR_REPLACE = 0;
  ADD_ONLY = 1;
  REPLACE_ONLY = 2;
  READ_ONLY = 0;
  READ_WRITE = 1;
  READ_ONLY_SHARE = 2;
  READ_ONLY_SHARED = 2;
  READ_WRITE_SHARE = 3;
  READ_WRITE_SHARED = 3;
  XNOFINDER = 1;

  XLOOKUP = 1;
  XNEXT = 2;
  XPREVIOUS = 4;
  XCURRENT = 8;
  XANY = 16;
  XEQ = 32;
  XBEGINS = 64;
  XNOT = 128;
  XLT = 256;
  XGT = 512;
  XLE = 1024;
  XGE = 2048;
  XGET_DATA = 4096;
  XNO_DATA = 8192;
  XNATIVE = 16384;
  XFAST_READ = -32768;

  NO_CACHE = -1; SMALL_CACHE = 0;	LARGE_CACHE = 1;

type

  // Define the TIsamInfoRec record

  TIsamInfoRec = record
    RecordsInFile:         Integer;
    GroupsUsed:            smallint;
    AllocatedGpsAvail:     smallint;
    MaxPrimaryKeyLen:      smallint;
    Pad1:                  smallint;
    GroupSize:             Integer;
    InitialAllocation:     smallint;
    IncrementalAllocation: smallint;
    FreeFileNoteSpace:     smallint;
    Pad2:                  smallint;
    StandardFormatAddr:    Integer;
    Reserved:              string;
  end;

  // Define pointer types for the VBIS4032 function parameters

  TStringPtr      = ^string;
  TLongIntPtr     = ^Integer;
  TSmallIntPtr    = ^smallint;
  TIsamInfoRecPtr = ^TIsamInfoRec;

  // Define the procedural types for the VBIS5032 functions

  TVmxCreate     = function (DatasetName: TStringPtr; MaxPrimaryKeyLen: TSmallIntPtr; GroupSize: TLongIntPtr; InitAlloc: TSmallIntPtr; Options: TSmallIntPtr; FormatStr: TStringPtr): smallint; stdcall;
  TVmxOpen       = function (DatasetName: TStringPtr; CacheSize: TSmallIntPtr; AccessMode: TSmallIntPtr; RDatasetNo: TLongIntPtr; LicenseKey: TStringPtr): smallint; stdcall;
  TVmxKill       = function (DatasetName: TStringPtr): smallint; stdcall;
  TVmxClose      = function (DatasetNo: TLongIntPtr): smallint; stdcall;
  TVmxGet        = function (DatasetNo: TLongIntPtr; Index: TSmallIntPtr; Options: TSmallIntPtr; Selector: TStringPtr; RIndexKey: TStringPtr; RPrimaryKey: TStringPtr; RawRecord: Pointer): smallint; stdcall;
  TVmxPut        = function (DatasetNo: TLongIntPtr; PrimaryKey: TStringPtr; RawRecord: pointer; UpdateMode: TSmallIntPtr): smallint; stdcall;
  TVmxDelete     = function (DatasetNo: TLongIntPtr; PrimaryKey: TStringPtr): smallint; stdcall;
  TVmxBOF        = function (DatasetNo: TLongIntPtr; Index: TSmallIntPtr): smallint; stdcall;
  TVmxEOF        = function (DatasetNo: TLongIntPtr; Index: TSmallIntPtr): smallint; stdcall;
  TVmxInfo       = function (DatasetNo: TLongIntPtr; RInfoStructure: TIsamInfoRecPtr): smallint; stdcall;
  TVmxLock       = function (DatasetNo: TLongIntPtr; LockString: TStringPtr): smallint; stdcall;
  TVmxUnlock     = function (DatasetNo: TLongIntPtr; LockString: TStringPtr): smallint; stdcall;
  TVmxReadNote   = function (DatasetNo: TLongIntPtr; NoteKey: TStringPtr; RContents: TStringPtr): smallint; stdcall;
  TVmxWriteNote  = function (DatasetNo: TLongIntPtr; NoteKey: TStringPtr; Contents: TStringPtr): smallint; stdcall;
  TVmxFetchField = function (DatasetNo: TLongIntPtr; FormatStr: TStringPtr; RawRecord: Pointer; FieldNum: tSmallIntPtr; FieldType: TStringPtr; RContents: tStringPtr): smallint; stdcall;
  TVmxStoreField = function (DatasetNo: TLongIntPtr; FormatStr: TStringPtr; RawRecord: Pointer; FieldNum: tSmallIntPtr; FieldType: tStringPtr; Contents: tStringPtr): smallint; stdcall;
  TVmxSetDictBof = function (DatasetNo: TLongIntPtr): smallint; stdcall;
  TVmxDeleteDict = function (DatasetNo: TLongIntPtr; DictKey: TStringPtr): smallint; stdcall;
  TVmxWriteDict  = function (DatasetNo: TLongIntPtr; DictKey: TStringPtr; Contents: TStringPtr): smallint; stdcall;
  TVmxReadDict   = function (DatasetNo: TLongIntPtr; DictKey: TStringPtr; RContents: TStringPtr; Options: TSmallIntPtr): smallint; stdcall;

  // Define the TIsamFile object

  TIsamFile = class(TObject)
  private
    fOpen: boolean;
    fHandle: Integer;
    fDataset: string;
    fVersion: string;
    fInitialised: boolean;

    function GetRecordCount: Integer;
    function GetFormatString: string;
    function GetPrimaryKeyLength: smallint;

  public

    constructor Create(const DllPath: string);
    destructor Destroy; override;

    function CreateNew(const FileName: string; const MaxPKeyLength: smallint; const FormatStr: string): Integer;
    // Remember to put your license key into this function
    function Open(const FileName: string; const AccessMode: smallint; LicenseKey: String = ''): Integer;
    function Kill(const FileName: string): Integer;
    function ReadNote(const Name: string; var Contents: string): Integer;
    function WriteNote(const Name: string; const Contents: string): Integer;
    function UnLock(const LString: string): Integer;
    function Lock(const LString: string): Integer;

    function Delete(const PKey: string): Integer;
    function Bof(const Index: smallint): Integer;
    function Eof(const Index: smallint): Integer;
    function Close: Integer;
    function Get(const Index: smallint; const Options: smallint; const Selector: string; var PrimaryKey, SecondaryKey: String; var RawRecord: Pointer): Integer;
    function Put(const Raw: pointer; const PKey: string): Integer;
    function StoreField(var RawRecord: Pointer; Const FieldNum: SmallInt; Const FieldData: String): Integer; Overload;
    function StoreField(var RawRecord: Pointer; Const FieldNum: SmallInt; Const FieldData: Integer): Integer; Overload;
    function FetchField(RawRecord: Pointer; Const FieldNum: SmallInt; var RetData: Integer): Integer; Overload;
    function FetchField(RawRecord: Pointer; Const FieldNum: SmallInt; var RetData: String): Integer; Overload;
    function SetDictBof: Integer;
    function DeleteDict(Const DictKey: String): Integer;
    function ReadDict(var DictKey: String; var Contents: String; const Options: smallint): Integer;
    function WriteDict(Const DictKey: String; Const Contents: String): Integer;

    // Public properties

    property Handle: Integer read fHandle;
    property FileName: string read fDataset;
    property Initialised: boolean read fInitialised;
    property RecordCount: Integer read GetRecordCount;
    property FormatString: string read GetFormatString;
    property PrimaryKeyLength: smallint read GetPrimaryKeyLength;
  end;

var

  // Placeholders for VBIS5032 functions

    VmxCreate:    TVmxCreate;
    VmxOpen:      TVmxOpen;
    VmxClose:     TVmxClose;
    VmxGet:       TVmxGet;
    VmxPut:       TVmxPut;
    VmxDelete:    TVmxDelete;
    VmxBOF:       TVmxBOF;
    VmxEOF:       TVmxEOF;
    VmxInfo:      TVmxInfo;
    VmxLock:      TVmxLock;
    VmxUnlock:    TVmxUnLock;
    VmxKill:      TVmxKill;
    VmxReadNote:  TVmxReadNote;
    VmxWriteNote: TVmxWriteNote;
    VmxFetchField: TVmxFetchField;
    VmxStoreField: TVmxFetchField;
    VmxSetDictBof: TVmxSetDictBof;
    VmxDeleteDict: TVmxDeleteDict;
    VmxReadDict: TVmxReadDict;
    VmxWriteDict: TVmxWriteDict;

implementation

uses
  StrUtils, Windows;

constructor TIsamFile.Create(const DllPath: string);
var
  HInstance: integer;
begin
  inherited Create;
  fInitialised := False;

  fVersion     := '1.00';
  fOpen        := False;

  HInstance := LoadLibrary(PChar(IncludeTrailingPathDelimiter(DllPath) + 'VBIS5032.DLL'));
  if HInstance <= HINSTANCE_ERROR then Exit;

  @VmxCreate     := GetProcAddress(HInstance, 'VmxCreate');
  @VmxOpen       := GetProcAddress(HInstance, 'VmxOpen');
  @VmxClose      := GetProcAddress(HInstance, 'VmxClose');
  @VmxGet        := GetProcAddress(HInstance, 'VmxGet');
  @VmxPut        := GetProcAddress(HInstance, 'VmxPut');
  @VmxDelete     := GetProcAddress(HInstance, 'VmxDelete');
  @VmxBOF        := GetProcAddress(HInstance, 'VmxBOF');
  @VmxEOF        := GetProcAddress(HInstance, 'VmxEOF');
  @VmxInfo       := GetProcAddress(HInstance, 'VmxInfo');
  @VmxLock       := GetProcAddress(HInstance, 'VmxLock');
  @VmxUnlock     := GetProcAddress(HInstance, 'VmxUnlock');
  @VmxKill       := GetProcAddress(HInstance, 'VmxKill');
  @VmxReadNote   := GetProcAddress(HInstance, 'VmxReadNote');
  @VmxWriteNote  := GetProcAddress(HInstance, 'VmxWriteNote');
  @VmxFetchField := GetProcAddress(HInstance, 'VmxFetchField');
  @VmxStoreField := GetProcAddress(HInstance, 'VmxStoreField');
  @VmxSetDictBof := GetProcAddress(HInstance, 'VmxSetDictBof');
  @VmxDeleteDict := GetProcAddress(HInstance, 'VmxDeleteDict');
  @VmxReadDict   := GetProcAddress(HInstance, 'VmxReadDict');
  @VmxWriteDict  := GetProcAddress(HInstance, 'VmxWriteDict');

  if (@VmxCreate     = nil) or
     (@VmxOpen       = nil) or
     (@VmxClose      = nil) or
     (@VmxGet        = nil) or
     (@VmxPut        = nil) or
     (@VmxDelete     = nil) or
     (@VmxBOF        = nil) or
     (@VmxEOF        = nil) or
     (@VmxInfo       = nil) or
     (@VmxLock       = nil) or
     (@VmxUnlock     = nil) or
     (@VmxKill       = nil) or
     (@VmxReadNote   = nil) or
     (@VmxFetchField = nil) or
     (@VmxStoreField = nil) or
     (@VmxSetDictBof = nil) or
     (@VmxDeleteDict = nil) or
     (@VmxReadDict   = nil) or
     (@VmxWriteDict  = nil) or
     (@VmxWriteNote  = nil) then Exit;

  fInitialised := True;
end;

destructor TIsamFile.Destroy;
begin
  Self.Close;
  inherited Destroy;
end;

function TIsamFile.CreateNew(const FileName: string; const MaxPKeyLength: smallint;
  const FormatStr: string): Integer;
var
  GpSz: Integer;
  Options: smallint;
  InitAlloc: smallint;
begin
  GpSz           := 0;
  InitAlloc      := 0;
  Options        := 0;

  Result := VmxCreate(@FileName, @MaxPKeyLength, @GpSz, @InitAlloc,
                @Options, @FormatStr);
end;

function TIsamFile.WriteDict(Const DictKey: String; Const Contents: String): Integer;
begin
  Result := VmxWriteDict(@fHandle, @DictKey, @Contents);
end;

function TIsamFile.ReadDict(var DictKey: String; var Contents: String;
  const Options: smallint): Integer;
var
  rContents: Integer;
begin
  rContents := 0;

  Result := VmxReadDict(@fHandle, @DictKey, @rContents, @Options);

  Contents := String(PChar(rContents));
  SysFreeString(Pointer(rContents));
end;

function TIsamFile.DeleteDict(Const DictKey: String): Integer;
begin
  Result := VmxDeleteDict(@fHandle, @DictKey);
end;

function TIsamFile.SetDictBof: Integer;
begin
  Result := VmxSetDictBof(@fHandle);
end;

function TIsamFile.GetRecordCount: Integer;
var
  IsamInfoRec: TIsamInfoRec;
  x: Integer;
begin
  IsamInfoRec.StandardFormatAddr := 0;
  x := VmxInfo(@fHandle, @IsamInfoRec);

  if x = VIS_OK then Result := IsamInfoRec.RecordsInFile
    else Result := -1;

  SysFreeString(Pointer(IsamInfoRec.StandardFormatAddr));
end;

function TIsamFile.GetFormatString: string;
var
  IsamInfoRec: TIsamInfoRec;
  x: Integer;
begin
  IsamInfoRec.StandardFormatAddr := 0;
  x := VmxInfo(@fHandle, @IsamInfoRec);

  if x = VIS_OK then
    Result := String(PChar(IsamInfoRec.StandardFormatAddr))
      else Result := '';

  SysFreeString(Pointer(IsamInfoRec.StandardFormatAddr));
end;

function TIsamFile.GetPrimaryKeyLength: smallint;
var
  IsamInfoRec: TIsamInfoRec;
  x: Integer;
begin
  IsamInfoRec.StandardFormatAddr := 0;
  x := VmxInfo(@fHandle, @IsamInfoRec);

  if x = VIS_OK then Result := IsamInfoRec.MaxPrimaryKeyLen
    else Result := -1;

  SysFreeString(Pointer(IsamInfoRec.StandardFormatAddr));
end;
 
// Remember to put your license key into this function
function TIsamFile.Open(const FileName: string; const AccessMode: smallint;
  LicenseKey: String = ''): Integer;
var
  CacheSize: smallint;
begin
  CacheSize   := SMALL_CACHE;

  Result := VmxOpen(@FileName, @CacheSize, @AccessMode, @fHandle, @LicenseKey);

  if Result = VIS_OK then
    begin
      fOpen := True;
      fDataset := Filename;
    end;
end;

function TIsamFile.Close: Integer;
begin
  Result := VmxClose(@fHandle);

  if Result = VIS_OK then
    begin
      fHandle := 0;
      fOpen := False;
      fDataset := '';
    end;
end;

function TIsamFile.Put(Const Raw: pointer; const PKey: string): Integer;
var
  UpdateMode: smallint;
begin
  UpdateMode := ADD_OR_REPLACE + XNATIVE;
  Result := VmxPut(@fHandle, @PKey, @Raw, @UpdateMode);
end;

function TIsamFile.StoreField(var RawRecord: Pointer; Const FieldNum: SmallInt;
  Const FieldData: Integer): Integer;
var
  FieldDef, FieldType: String;
begin
  FieldDef := '';
  FieldType := '%';

  Result := vmxStoreField(@fHandle, @FieldDef, @RawRecord,
    @FieldNum, @FieldType, @FieldData);
end;

function TIsamFile.StoreField(var RawRecord: Pointer; Const FieldNum: SmallInt;
  Const FieldData: String): Integer;
var
  FieldDef, FieldType: String;
begin
  FieldDef := '';
  FieldType := '$';

  Result := vmxStoreField(@fHandle, @FieldDef, @RawRecord,
    @FieldNum, @FieldType, @FieldData);
end;

function TIsamFile.FetchField(RawRecord: Pointer; Const FieldNum: SmallInt;
  var RetData: Integer): Integer;
var
  FieldDef, FieldType: String;
  rField: Integer;
begin
  rField := 0;
  FieldDef := '';
  FieldType := '%';
  RetData := 0;

  Result := vmxFetchField(@fHandle, @FieldDef, @RawRecord,
    @FieldNum, @FieldType, @rField);

  RetData := Integer(rField);
end;

function TIsamFile.FetchField(RawRecord: Pointer; Const FieldNum: SmallInt;
  var RetData: String): Integer;
var
  FieldDef, FieldType: String;
  rField: Integer;
begin
  rField := 0;
  FieldDef := '';
  FieldType := '$';
  RetData := '';

  Result := vmxFetchField(@fHandle, @FieldDef, @RawRecord,
    @FieldNum, @FieldType, @rField);

  RetData := String(PChar(rField));
  SysFreeString(Pointer(rField));
end;

function TIsamFile.Get(const Index: smallint; const Options: smallint;
  const Selector: string; var PrimaryKey, SecondaryKey: String;
  var RawRecord: Pointer): Integer;
var
  RIndexEntry: Integer;
  RPrimaryKey: Integer;
begin

  RIndexEntry := 0;
  RPrimaryKey := 0;
  RawRecord := nil;

  Result := VmxGet(@fHandle, @Index, @Options, @Selector,
              @RIndexEntry, @RPrimaryKey, @RawRecord);

  PrimaryKey := String(PChar(RPrimaryKey));
  SecondaryKey := String(PChar(RIndexEntry));

  SysFreeString(Pointer(RIndexEntry));
  SysFreeString(Pointer(RPrimaryKey));

end;

function TIsamFile.Delete(const PKey: string): Integer;
begin
  Result := VmxDelete(@fHandle, @PKey);
end;

function TIsamFile.Bof(const Index: smallint): Integer;
begin
  Result := VmxBof(@fHandle, @Index);
end;

function TIsamFile.Eof(const Index: smallint): Integer;
begin
  Result := VmxEof(@fHandle, @Index);
end;

function TIsamFile.Lock(const LString: string): Integer;
begin
  Result := VmxLock(@fHandle, @LString);
end;

function TIsamFile.UnLock(const LString: string): Integer;
begin
  Result := VmxUnlock(@fHandle, @LString);
end;

function TIsamFile.Kill(const FileName: string): Integer;
begin
  Result := VmxKill(@FileName);
end;

function TIsamFile.WriteNote(const Name: string; const Contents: string): Integer;
begin
  Result := VmxWriteNote(@fHandle, @Name, @Contents);
end;

function TIsamFile.ReadNote(const Name: string; var Contents: string): Integer;
var
  RContents: Integer;
begin
  Result := VmxReadNote(@fHandle, @Name, @RContents);
  Contents := String(PChar(RContents));
  SysFreeString(Pointer(RContents));
end;

end.
