// structure definitions

#   define PAREA_SIZE       996

typedef struct tagFINDER    // finder record
  {
    short       rcont;      // record continuation status
                            //   1 to (n-1) = continued parts of record
                            //           -n = last part of continued record
    char        key[1];     // first key in group

  } FINDER;
typedef FINDER FAR *LPFINDER;

typedef struct tagLOOKUP    // group lookup entry
  {
    LONG        group;      // logical group number
    short       rcont;      // record continuation status
                            //   1 to (n-1) = continued parts of record
                            //           -n = last part of continued record
    char        key[1];     // first key in group

  } LOOKUP;
typedef LOOKUP FAR *LPLOOKUP;

typedef struct tagTDMILL    // trailer for a WndMill file
  {
    char        sigid[8];   // place to put 'VB/ISAM' signature
    LONG        chgcnt;     // changed when split/collapse
    WORD        grp_size;   // size of each group in this file partition
    WORD        grp_count;  // count of groups in file
    WORD        ext_count;  // count of extensions in file
    WORD        ft_flags;   // file type flags
    LONG        map_offset; // offset of map in main extension
    LONG        ext_offset; // offset of extension map in main extension
    WORD        finder_len; // length of finder items
    WORD        init_alloc; // number of groups to allocate for base file
    WORD        cr_opts;    // create_options
    WORD        free_alloc; // number of free groups in current allocation
    LONG        nRecords;   // total number of records in database

    short       pareaused;         // space used in props area
    short       pareasize;         // current size of parea
    char        parea[PAREA_SIZE]; // property area

    // NOTE: space for following variables stolen from end of parea
    //       by reducing it's size.

    DWORD       tp[4];  						// temp

    LONG        nChgCount;         // current change count of dataset
    LONG        nMapGeneration;    // current generation of map
    LONG        nPrimRecs;         // number of primary records for MX

  } TDMILL;
typedef TDMILL FAR *LPTDMILL;

typedef struct tagMAPENT    // map entry
  {
    union
      {
        char    cv[4];      // cv[3] has extent number
        LONG    lv;         // lv & 0x00ffffff has group number

      } me;

  } MAPENT;
typedef MAPENT FAR *LPMAPENT;

typedef struct tagMREC      // header for each record in a WndMill file
  {
    WORD        rsize;      // size of record
    short       rcont;      // record continuation status
                            //   1 to (n-1) = continued parts of record
                            //           -n = last part of continued record
    BYTE        ksize;      // size of the key in this record
    char        key[1];     // start of key + data for the record

  } MREC;
typedef MREC FAR *LPMREC;

typedef struct tagGROUP     // header for each wndmill file group
  {
    WORD        gchksum;    // group checksum (not yet used)
    WORD        glen;       // length of data in group including header
    MREC        first_rec;  // first record in group;

  } GROUP;
typedef GROUP FAR *LPGROUP;

typedef struct tagXREFS
  {
    short       fldno;      // field number in record
    short       mwidth;     // max field width for this key
    char        fldtype;    // field type (' ' var str, '*' fixed str)

    char        pckey[1];   // prefix char for items in this xref
    char        ckey[255];  // key at current positon in this xref
    short       inexact;    // inbetween records

    MAPENT      r_grp_map;  // map entry of logical group (where found)
    WORD        r_goffset;  // offset of record read

    char        pokey[1];   // prefix char for items in this xref
    char        okey[255];  // old key in this xref (for update)

    char        pnkey[1];   // prefix char for items in this xref
    char        nkey[255];  // new key in this xref (for update)

    short       nflds;
    short       fldnos[MAX_CFLD_NOS];
    short       fldwids[MAX_CFLD_NOS];

  } XREFS;
typedef XREFS FAR *LPXREFS;

typedef struct tagLOCKENT
  {
    DWORD       hash_value; // hashed value of key
    HANDLE      hlock_key;  // text value of semaphore key

  } LOCKENT;
typedef LOCKENT FAR *LPLOCKENT;


typedef struct tagWMCB      // WndMill control block
  {
    HANDLE      hwmcb;      // handle of this control block
    short       mode;       // mode of LO_READ, LO_WRITE, or LO_READWRITE
    short       share;      // if opened for VMODE_RO_SHARE or VMODE_RW_SHARE

    short       svmode;     // sharing availability mode (1, 2, 3)

    short       is_nt;      // set if open in NT mode

    short       dict;       // 1 = dict portion, 0 = data portion

    char        name[MAXX_PATH];  // dos file, drive, path, name and ext (.ISD)
    HFILE       fh;         // file handle for open file (.ISD)

    char        fname[MAXX_PATH]; // dos file, drive, path, name and ext (.ISF)
    HFILE       fhf;        // file handle for open finder file (.ISF)

    char        mname[MAXX_PATH]; // dos file, drive, path, name and ext (.ISM)
    HFILE       fhm;        // file handle for open map file (.ISM)

    HFILE       fhl;        // file handle for open lock file (.ISL)

    short       drive;      // drive number 0=? 1=A 2=B 3=C ...

    short       fk_mod;     // flag to indicate first key in a group changed

    HANDLE      hgrpbuf;    // handle to current group buffer
    LPGROUP     lpgroup;    // pointer to the current group buffer
    LPMREC      lpmrec;     // pointer to the record in group buffer
    WORD        goffset;    // offset of last record read

    WORD        log_gnum;   // logical group number of current group buffer
    MAPENT      grp_map;    // map entry of logical group
    WORD        grp_dirty;  // group in memory has been modified
    WORD        tdm_dirty;  // tdmill in memory has been modified

    TDMILL      mapinfo;    // info on map loc and size

    short       status;     // status flags

    HANDLE      hmap;       // handle of map in memory if loaded
    LPMAPENT    lpmap;      // pointer to map in memory if loaded and locked

    WORD        mLookupsOrig;   // original lookup parm // added don 8-17-93

    WORD        mLookups;   // maximum number of lookups in lookup table
    WORD        cLookups;   // number of lookups in lookup table
    HANDLE      hLookups;   // handle of lookup table
    WORD        lLookent;   // length of a lookup entry

    LONG        nWrites;    // total number of actual writes
    LONG        nReads;     // total number of actual reads

    short       fInexact;   // flag indicating at an inexact position

    WORD        read_write; // set if database opened for read/write

    short       rec_split;  // set if current record had to be split

    // for mk

    short       mke;           // flag to indicate this is a mke dataset
    short       nflds;         // number of fields in record  // added 9-3-92
    short       nxrefs;        // number of xrefs
    HANDLE      hxrefs;        // handle to xrefs
    LPXREFS     lpxrefs;       // pointer to xrefs

    short       lencurfmt;     // length of format string for this file
    char        curfmt[256];   // format string for this file
    WORD        sigfmt;        // 'signature' of format string
    WORD        len_empty_rec; // length of an 'empty' record
    WORD        len_type;      // length of a type structure

    short       valid_section; // section of file last accessed and therefore
                               // that can be accessed again without
                               // repositioning

    char        fxlate[256];   // handle to field number to index number xlate

    short       tint;          // temp variables (scratch)
    WORD        tword;         // ...
    LONG        tlong;         // ...

    WORD        nlockents;     // current number of lock entries
    LOCKENT     lockents[MAX_LOCK_ENTS];    // lock entry table

    short       exterror;      // extended error code for VmxExtendedError()
    short       Map_Is_Locked; // indicates a lock at 0 offset is set

    short       has_vl_str;    // flag to indicate record has variable len str(s)

    short       no_hi_bit;     // set when hi bit a problem with lock offsets

    HANDLE      busy_lockout;  // lockout flag for 'long' funcitons

    short       d_fast;  // Option flags for XGET_FAST

    int         file_encrypted; // indicates file is encrypted 
    DWORD       Tp[4];          // temp      

  } WMCB;
typedef WMCB FAR *LPWMCB;
